# -*- coding: utf-8 -*-
# Generated by Django 1.9.10 on 2017-04-03 06:07
from __future__ import unicode_literals

from django.db import migrations
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('simple_locations', '0004_auto_20170330_1618'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='area',
            managers=[
                ('tree', django.db.models.manager.Manager()),
            ],
        ),
    ]
