# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-03-05 08:26
from __future__ import unicode_literals

import django.contrib.gis.db.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('simple_locations', '0006_auto_20180202_1540'),
    ]

    operations = [
        migrations.AlterField(
            model_name='area',
            name='geom',
            field=django.contrib.gis.db.models.fields.MultiPolygonField(blank=True, null=True, srid=4326),
        ),
    ]
