# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-02-02 06:40
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('simple_locations', '0005_auto_20170403_1507'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='area',
            managers=[
            ],
        ),
    ]
