# Generated by Django 3.1.13 on 2022-02-20 16:03

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("simple_locations", "0010_add_border_model"),
    ]

    operations = [
        migrations.AddField(
            model_name="border",
            name="area_ids",
            field=django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), default=list, size=None),
        ),
        migrations.AddField(
            model_name="border",
            name="area_types",
            field=django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), default=list, size=None),
        ),
    ]
