# Generated by Django 3.1.14 on 2022-03-07 06:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('simple_locations', '0012_auto_20220221_0817'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='projectedarea',
            name='id',
        ),
        migrations.AlterField(
            model_name='area',
            name='level',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='area',
            name='lft',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='area',
            name='rght',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='projectedarea',
            name='area',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, serialize=False, to='simple_locations.area'),
        ),
    ]
